@@ -42,14 +42,14 @@
 arg_pat="$1"
 
 # the version of this script
-bm_version=3.20.0
+bm_version=4.2.0
 
 # CONFIGURABLE VARIABLES ---------------------------------------
 # change as needed to reflect your environment/workloads
 
 # the path to the qsv binary, change this if you're not using the prebuilt binaries
 # e.g. you compiled a tuned version of qsv with different features and/or CPU optimizations enabled
-# qsv_bin=../target/release/qsv
+# qsv_bin=../target/release/qsvlite
 # qsv_bin=../target/debug/qsv
 qsv_bin=qsv
 # the path to the qsv binary that we dogfood to run the benchmarks
@@ -148,6 +148,7 @@ if [[ "$arg_pat" == "setup" ]]; then
   need_hyperfine=0
   need_awk=0
   need_sed=0
+  need_duckdb=0
 
   # check if 7z is installed
   if ! command -v "$sevenz_bin" &>/dev/null; then
@@ -169,6 +170,11 @@ if [[ "$arg_pat" == "setup" ]]; then
     need_sed=1
   fi
 
+  # check if duckdb is installed
+  if ! command -v duckdb &>/dev/null; then
+    need_duckdb=1
+  fi
+
   # if all required tools are installed, exit
   if [[ "$need_sevenz" -eq 0 && "$need_hyperfine" -eq 0 && "$need_awk" -eq 0 && "$need_sed" -eq 0 ]]; then
     echo "> All required tools are installed..."
@@ -206,6 +212,12 @@ if [[ "$arg_pat" == "setup" ]]; then
     brew install gnu-sed
   fi
 
+  # if duckdb is not installed, install it
+  if [[ "$need_duckdb" -eq 1 ]]; then
+    echo "INFO: duckdb could not be found. Installing..."
+    brew install duckdb
+  fi
+
   echo "> All required tools installed! You can run ./benchmarks.sh now."
   exit
 fi
@@ -238,6 +250,13 @@ if ! command -v sed &>/dev/null; then
   exit
 fi
 
+# check if duckdb is installed
+if ! command -v duckdb &>/dev/null; then
+  echo "ERROR: duckdb could not be found"
+  echo "Please install duckdb or run \"./benchmarks.sh setup\" to install it."
+  exit
+fi
+
 # qsv version metadata ----------------
 version=$(echo "$raw_version" | cut -d' ' -f2 | cut -d'-' -f1)
 # get target platform from version
@@ -452,6 +471,7 @@ run enum_uuid "$qsv_bin" enum --uuid "$data"
 run enum_constant "$qsv_bin" enum --constant "NYC" "$data"
 run enum_copy "$qsv_bin" enum --copy Agency "$data"
 run excel "$qsv_bin" excel benchmark_data.xlsx
+run excel_error_format_formula "$qsv_bin" excel --error-format both benchmark_data.xlsx
 run excel_metadata "$qsv_bin" excel --metadata c benchmark_data.xlsx
 run excel_metadata_short "$qsv_bin" excel --metadata s benchmark_data.xlsx
 run exclude "$qsv_bin" exclude \'Incident Zip\' "$data" \'Incident Zip\' data_to_exclude.csv
@@ -472,8 +492,13 @@ run flatten_condensed "$qsv_bin" flatten "$data" --condense 50
 run fmt "$qsv_bin" fmt --crlf "$data"
 run fmt_no_crlf "$qsv_bin" fmt "$data"
 run fmt_no_final_newline "$qsv_bin" fmt --no-final-newline "$data"
+run foreach "$qsv_bin" foreach City "echo {}" "$data"
 run frequency "$qsv_bin" frequency "$data"
 run --index frequency_index "$qsv_bin" frequency "$data"
+run frequency_no_limit "$qsv_bin" frequency --limit 0 "$data"
+run --index frequency_no_limit_index "$qsv_bin" frequency --limit 0 "$data"
+run frequency_other_sorted "$qsv_bin" frequency --other-sorted "$data"
+run --index frequency_other_sorted_index "$qsv_bin" frequency --other-sorted "$data"
 run frequency_selregex "$qsv_bin" frequency -s /^R/ "$data"
 run frequency_j1 "$qsv_bin" frequency -j 1 "$data"
 run frequency_ignorecase "$qsv_bin" frequency -i "$data"
@@ -535,7 +560,11 @@ run searchset_unicode "$qsv_bin" searchset searchset_patterns_unicode.txt --unic
 run select "$qsv_bin" select \'Agency,Community Board\' "$data"
 run select_regex "$qsv_bin" select /^L/ "$data"
 run slice_one_middle "$qsv_bin" slice -i 500000 "$data"
+run slice_last_1k "$qsv_bin" slice -s 1000 -l 1000 "$data"
+run slice_last_1k_json "$qsv_bin" slice -s 1000 -l 1000 --json "$data"
 run --index slice_one_middle_index "$qsv_bin" slice -i 500000 "$data"
+run --index slice_last_1k_index "$qsv_bin" slice -s 1000 -l 1000 "$data"
+run --index slice_last_1k_json_index "$qsv_bin" slice -s 1000 -l 1000 --json "$data"
 run snappy_compress "$qsv_bin" snappy compress "$data" --output benchmark_data.snappy
 run snappy_decompress "$qsv_bin" snappy decompress benchmark_data.snappy
 run snappy_validate "$qsv_bin" snappy validate benchmark_data.snappy
@@ -555,6 +584,9 @@ run --index split_chunks_index "$qsv_bin" split --chunks 20 split_tempdir_chunks
 run --index split_chunks_index_j1 "$qsv_bin" split --chunks 20 -j 1 split_tempdir_chunks_idx_j1
 run sqlp "$qsv_bin" sqlp "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_aggregations "$qsv_bin" sqlp "$data" -Q '"select Borough, count(*) from _t_1 where \"Complaint Type\"='\''Noise'\'' group by Borough"'
+run sqlp_aggregations_vs_duckdb duckdb :memory: '"select Borough, count(*) from read_csv_auto('\'''$data''\'') where \"Complaint Type\"='\''Noise'\'' group by Borough"'
+run sqlp_aggregations_expensive "$qsv_bin" sqlp SKIP_INPUT -Q expensive.sql
+run sqlp_aggregations_expensive_vs_duckdb duckdb :memory: -c \".read expensiveduckdb.sql\"
 run sqlp_format_arrow "$qsv_bin" sqlp --format arrow "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_format_avro "$qsv_bin" sqlp --format avro "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
 run sqlp_format_json "$qsv_bin" sqlp --format json "$data" -Q '"select * from _t_1 where \"Complaint Type\"='\''Noise'\'' and Borough='\''BROOKLYN'\''"'
@@ -596,8 +628,10 @@ run transpose "$qsv_bin" transpose "$data"
 run transpose_multipass "$qsv_bin" transpose --multipass "$data"
 run validate "$qsv_bin" validate "$data" "$schema"
 run validate_no_schema "$qsv_bin" validate "$data"
+run validate_valid_output "$qsv_bin" validate "$data" "$schema" --valid-output -
 run --index validate_index "$qsv_bin" validate "$data" "$schema"
 run --index validate_no_schema_index "$qsv_bin" validate "$data"
+run --index validate_valid_output_index "$qsv_bin" validate "$data" "$schema" --valid-output -
 
 # show count of commands to be benchmarked
 with_index_count=${#commands_with_index[@]}
