@@ -232,3 +232,15 @@ impl From<polars::error::PolarsError> for CliError {
         CliError::Other(format!("Polars error: {err:?}"))
     }
 }
+
+impl From<flexi_logger::FlexiLoggerError> for CliError {
+    fn from(err: flexi_logger::FlexiLoggerError) -> CliError {
+        CliError::Other(format!("FlexiLogger error: {err:?}"))
+    }
+}
+
+impl From<chrono_tz::ParseError> for CliError {
+    fn from(err: chrono_tz::ParseError) -> CliError {
+        CliError::Other(format!("ChronoTZ error: {err:?}"))
+    }
+}
