@@ -2,14 +2,12 @@ static USAGE: &str = r#"
 Apply a series of transformation functions to given CSV column/s. This can be used to
 perform typical data-wrangling tasks and/or to harmonize some values, etc.
 
-It has five subcommands:
- 1. operations*   - 36 string, format, currency, regex & NLP operators.
+It has four subcommands:
+ 1. operations*   - 37 string, format, currency, regex & NLP operators.
  2. emptyreplace* - replace empty cells with <--replacement> string.
- 3. datefmt*      - Formats recognized date/s (19 formats recognized) to
-                    a specified date format using <--formatstr>.
- 4. dynfmt        - Dynamically constructs a new column from other columns using
+ 3. dynfmt        - Dynamically constructs a new column from other columns using
                     the <--formatstr> template.
- 5. calcconv      - parse and evaluate math expressions, with support for units
+ 4. calcconv      - parse and evaluate math expressions, with support for units
                     and conversions.
     * subcommand is multi-column capable.
 
@@ -27,7 +25,8 @@ number of transformed columns with the --rename option is the same. e.g.:
 
 # trim and fold to uppercase the col1,col2 and col3 columns and rename them
 # to newcol1,newcol2 and newcol3
-$ qsv apply operations trim,upper col1,col2,col3 -r newcol1,newcol2,newcol3 file.csv
+
+ $ qsv apply operations trim,upper col1,col2,col3 -r newcol1,newcol2,newcol3 file.csv
 
 It has 36 supported operations:
 
@@ -73,6 +72,7 @@ It has 36 supported operations:
       with --comparand. Automatically rounds values to two decimal places. Specify
       "euro" formatting (e.g. 1.000,00 instead of 1,000.00 ) by setting --formatstr to "euro".
       Specify conversion rate by setting --replacement to a number.
+  * gender_guess: Guess the gender of a name.
   * copy: Mark a column for copying
   * simdl: Damerau-Levenshtein similarity to --comparand
   * simdln: Normalized Damerau-Levenshtein similarity to --comparand (between 0.0 & 1.0)
@@ -168,44 +168,6 @@ case insensitive with 'None'.
 
 $ qsv apply emptyreplace --replacement None '/(?i)^observation/' file.csv
 
-DATEFMT (multi-column capable)
-Formats recognized date/s to a specified format using <--formatstr>.
-See https://github.com/jqnatividad/belt/tree/main/dateparser#accepted-date-formats for
-recognized date formats.
-See https://docs.rs/chrono/latest/chrono/format/strftime/ for 
-accepted date formats for --formatstr.
-Defaults to ISO 8601/RFC 3339 format when --formatstr is not specified.
-( "%Y-%m-%dT%H:%M:%S%z" - e.g. 2001-07-08T00:34:60.026490+09:30 )
-
-Examples:
-Format dates in Open Date column to ISO 8601/RFC 3339 format:
-
-  $ qsv apply datefmt 'Open Date' file.csv
-
-Format multiple date columns in file.csv to ISO 8601/RFC 3339 format:
-
-  $ qsv apply datefmt 'Open Date,Modified Date,Closed Date' file.csv
-
-Format all columns that end with "_date" case-insensitive in file.csv to ISO 8601/RFC 3339 format:
-
-  $ qsv apply datefmt '\(?i)_date$\' file.csv
-
-Format dates in OpenDate column using '%Y-%m-%d' format:
-
-  $ qsv apply datefmt OpenDate --formatstr '%Y-%m-%d' file.csv
-
-Format multiple date columns using '%Y-%m-%d' format:
-
-  $ qsv apply datefmt OpenDate,CloseDate,ReopenDate --formatstr '%Y-%m-%d' file.csv
-
-Get the week number for OpenDate and store it in the week_number column:
-
-  $ qsv apply dateformat OpenDate --formatstr '%V' --new-column week_number file.csv
-
-Get the day of the week for several date columns and store it in the corresponding weekday columns:
-
-  $ qsv apply dateformat OpenDate,CloseDate --formatstr '%u' --rename Open_weekday,Close_weekday file.csv
-
 DYNFMT
 Dynamically constructs a new column from other columns using the <--formatstr> template.
 The template can contain arbitrary characters. To insert a column value, enclose the
@@ -265,7 +227,6 @@ For more extensive examples, see https://github.com/jqnatividad/qsv/blob/master/
 Usage:
 qsv apply operations <operations> [options] <column> [<input>]
 qsv apply emptyreplace --replacement=<string> [options] <column> [<input>]
-qsv apply datefmt [--formatstr=<string>] [options] <column> [<input>]
 qsv apply dynfmt --formatstr=<string> [options] --new-column=<name> [<input>]
 qsv apply calcconv --formatstr=<string> [options] --new-column=<name> [<input>]
 qsv apply --help
@@ -273,7 +234,7 @@ qsv apply --help
 apply arguments:
     <column>                        The column/s to apply the transformation to.
                                     Note that the <column> argument supports multiple columns
-                                    for the operations, emptyreplace & datefmt subcommands.
+                                    for the operations & emptyreplace subcommands.
                                     See 'qsv select --help' for the format details.
 
     OPERATIONS subcommand:
@@ -284,12 +245,6 @@ apply arguments:
         --replacement=<string>      The string to use to replace empty values.
         <column>                    The column/s to check for emptiness.
 
-    DATEFMT subcommand:
-        --formatstr=<string>        The date format to use for the datefmt operation.
-                                    See DATEFMT section in the --formatstr option below
-                                    for more details.
-        <column>                    The date column/s to apply the datefmt operation to.
-
     DYNFMT subcommand:
         --formatstr=<string>        The template to use for the dynfmt operation.
                                     See DYNFMT example above for more details.
@@ -308,10 +263,6 @@ apply options:
                                 Also used with numtocurrency operation to specify currency symbol.
     -R, --replacement=<string>  The string to use for the replace & emptyreplace operations.
                                 Also used with numtocurrency operation to conversion rate.
-    --prefer-dmy                Prefer to parse dates in dmy format. Otherwise, use mdy format.
-                                Only used with the DATEFMT subcommand.
-    --keep-zero-time            If a formatted date ends with "T00:00:00+00:00", keep the time
-                                instead of removing it. Only used with the DATEFMT subcommand.
     -f, --formatstr=<string>    This option is used by several subcommands:
 
                                 OPERATIONS: 
@@ -328,12 +279,6 @@ apply options:
                                   round
                                     The number of decimal places to round to (default: 3)
 
-                                DATEFMT: The date format to use. For formats, see
-                                  https://docs.rs/chrono/latest/chrono/format/strftime/
-                                  Default to ISO 8601 / RFC 3339 date & time format -
-                                  "%Y-%m-%dT%H:%M:%S%z" - e.g. 2001-07-08T00:34:60.026490+09:30
-                                  [default: %+]
-
                                 DYNFMT: the template to use to construct a new column.
 
     -j, --jobs <arg>            The number of jobs to run in parallel.
@@ -358,10 +303,10 @@ use cpc::{eval, units::Unit};
 use data_encoding::BASE64;
 use dynfmt::Format;
 use eudex::Hash;
+use gender_guesser::Gender;
 use indicatif::{ProgressBar, ProgressDrawTarget};
 use log::debug;
 use qsv_currency::Currency;
-use qsv_dateparser::parse_with_preference;
 use rayon::{
     iter::{IndexedParallelIterator, ParallelIterator},
     prelude::IntoParallelRefIterator,
@@ -402,6 +347,7 @@ enum Operations {
     Encode,
     Escape,
     Eudex,
+    Gender_Guess,
     Len,
     Lower,
     Ltrim,
@@ -433,27 +379,24 @@ enum Operations {
 
 #[derive(Deserialize)]
 struct Args {
-    arg_column:          SelectColumns,
-    cmd_operations:      bool,
-    arg_operations:      String,
-    cmd_datefmt:         bool,
-    cmd_dynfmt:          bool,
-    cmd_emptyreplace:    bool,
-    cmd_calcconv:        bool,
-    arg_input:           Option<String>,
-    flag_rename:         Option<String>,
-    flag_comparand:      String,
-    flag_replacement:    String,
-    flag_prefer_dmy:     bool,
-    flag_keep_zero_time: bool,
-    flag_formatstr:      String,
-    flag_batch:          u32,
-    flag_jobs:           Option<usize>,
-    flag_new_column:     Option<String>,
-    flag_output:         Option<String>,
-    flag_no_headers:     bool,
-    flag_delimiter:      Option<Delimiter>,
-    flag_progressbar:    bool,
+    arg_column:       SelectColumns,
+    cmd_operations:   bool,
+    arg_operations:   String,
+    cmd_dynfmt:       bool,
+    cmd_emptyreplace: bool,
+    cmd_calcconv:     bool,
+    arg_input:        Option<String>,
+    flag_rename:      Option<String>,
+    flag_comparand:   String,
+    flag_replacement: String,
+    flag_formatstr:   String,
+    flag_batch:       u32,
+    flag_jobs:        Option<usize>,
+    flag_new_column:  Option<String>,
+    flag_output:      Option<String>,
+    flag_no_headers:  bool,
+    flag_delimiter:   Option<Delimiter>,
+    flag_progressbar: bool,
 }
 
 static CENSOR: OnceLock<Censor> = OnceLock::new();
@@ -483,7 +426,6 @@ static INDIANCOMMA_POLICY: SeparatorPolicy = SeparatorPolicy {
 #[derive(PartialEq)]
 enum ApplySubCmd {
     Operations,
-    DateFmt,
     DynFmt,
     EmptyReplace,
     CalcConv,
@@ -501,6 +443,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let headers = rdr.byte_headers()?.clone();
     let sel = rconfig.selection(&headers)?;
+    // safety: we just checked that sel is not empty in the previous line
     let column_index = *sel.iter().next().unwrap();
 
     let mut headers = rdr.headers()?.clone();
@@ -533,6 +476,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // first, get the fields used in the dynfmt template
         let formatstr_re: &'static Regex = crate::regex_oncelock!(r"\{(?P<key>\w+)?\}");
         for format_fields in formatstr_re.captures_iter(&args.flag_formatstr) {
+            // safety: we already checked that the regex match is valid
             dynfmt_fields.push(format_fields.name("key").unwrap().as_str());
         }
         // we sort the fields so we can do binary_search
@@ -567,8 +511,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             Err(e) => return Err(e),
         }
         ApplySubCmd::Operations
-    } else if args.cmd_datefmt {
-        ApplySubCmd::DateFmt
     } else if args.cmd_dynfmt {
         ApplySubCmd::DynFmt
     } else if args.cmd_emptyreplace {
@@ -590,7 +532,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // we set it to empty string
     let flag_replacement = if apply_cmd == ApplySubCmd::Operations
         && ops_vec.contains(&Operations::Regex_Replace)
-        && args.flag_replacement.to_lowercase() == NULL_VALUE
+        && args.flag_replacement.to_ascii_lowercase() == NULL_VALUE
     {
         String::new()
     } else {
@@ -611,8 +553,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         progress.set_draw_target(ProgressDrawTarget::hidden());
     }
 
-    let prefer_dmy = args.flag_prefer_dmy || rconfig.get_dmy_preference();
-
     // amortize memory allocation by reusing record
     #[allow(unused_assignments)]
     let mut batch_record = csv::StringRecord::new();
@@ -633,7 +573,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             match rdr.read_record(&mut batch_record) {
                 Ok(has_data) => {
                     if has_data {
-                        batch.push(batch_record.clone());
+                        batch.push(std::mem::take(&mut batch_record));
                     } else {
                         // nothing else to add to batch
                         break;
@@ -679,32 +619,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         for col_index in &*sel {
                             record[*col_index].clone_into(&mut cell);
                             if cell.trim().is_empty() {
-                                cell = flag_replacement.clone();
-                            }
-                            if flag_new_column.is_some() {
-                                record.push_field(&cell);
-                            } else {
-                                record = replace_column_value(&record, *col_index, &cell);
-                            }
-                        }
-                    },
-                    ApplySubCmd::DateFmt => {
-                        let mut cell = String::new();
-                        for col_index in &*sel {
-                            record[*col_index].clone_into(&mut cell);
-                            if !cell.is_empty() {
-                                let parsed_date = parse_with_preference(&cell, prefer_dmy);
-                                if let Ok(format_date) = parsed_date {
-                                    let formatted_date =
-                                        format_date.format(&flag_formatstr).to_string();
-                                    if !args.flag_keep_zero_time
-                                        && formatted_date.ends_with("T00:00:00+00:00")
-                                    {
-                                        cell = formatted_date[..10].to_string();
-                                    } else {
-                                        cell = formatted_date;
-                                    }
-                                }
+                                cell.clone_from(&flag_replacement);
                             }
                             if flag_new_column.is_some() {
                                 record.push_field(&cell);
@@ -999,6 +914,13 @@ fn validate_operations(
                 }
                 whatlang_invokes = whatlang_invokes.saturating_add(1);
             },
+            Operations::Gender_Guess => {
+                if flag_new_column.is_none() {
+                    return fail_incorrectusage_clierror!(
+                        "--new_column (-c) is required for Gender_Guess"
+                    );
+                }
+            },
             _ => {},
         }
         ops_vec.push(operation);
@@ -1083,6 +1005,18 @@ fn apply_operations(
                     Err(e) => format!("decoding error: {e:?}"),
                 };
             },
+            Operations::Gender_Guess => {
+                let gender_detector = gender_guesser::Detector::new();
+                *cell = match gender_detector.get_gender(cell) {
+                    Gender::Male => "Male".to_string(),
+                    Gender::Female => "Female".to_string(),
+                    Gender::MayBeMale => "MayBeMale".to_string(),
+                    Gender::MayBeFemale => "MayBeFemale".to_string(),
+                    Gender::BothMaleFemale => "BothMaleFemale".to_string(),
+                    Gender::NotSure => "NotSure".to_string(),
+                    Gender::NotFound => "NotFound".to_string(),
+                };
+            },
             Operations::Escape => {
                 *cell = cell.escape_default().to_string();
             },
@@ -1103,23 +1037,28 @@ fn apply_operations(
                 *cell = cell.replace(comparand, replacement);
             },
             Operations::Regex_Replace => {
+                // safety: we set REGEX_REPLACE in validate_operations()
                 let regexreplace = REGEX_REPLACE.get().unwrap();
                 *cell = regexreplace.replace_all(cell, replacement).to_string();
             },
             Operations::Censor => {
+                // safety: we set CENSOR in validate_operations()
                 let censor = CENSOR.get().unwrap();
                 *cell = censor.censor(cell);
             },
             Operations::Censor_Check => {
+                // safety: we set CENSOR in validate_operations()
                 let censor = CENSOR.get().unwrap();
                 *cell = censor.check(cell).to_string();
             },
             Operations::Censor_Count => {
+                // safety: we set CENSOR in validate_operations()
                 let censor = CENSOR.get().unwrap();
                 *cell = censor.count(cell).to_string();
             },
             Operations::Thousands => {
                 if let Ok(num) = cell.parse::<f64>() {
+                    //safety: we set THOUSANDS_POLICY in validate_operations()
                     let mut temp_string = num.separate_by_policy(*THOUSANDS_POLICY.get().unwrap());
 
                     // if there is a decimal separator (fractional part > 0.0), use the requested
@@ -1145,6 +1084,7 @@ fn apply_operations(
             },
             Operations::Round => {
                 if let Ok(num) = cell.parse::<f64>() {
+                    // safety: we set ROUND_PLACES in validate_operations()
                     *cell = util::round_num(num, *ROUND_PLACES.get().unwrap());
                 }
             },
@@ -1215,11 +1155,13 @@ fn apply_operations(
             },
             Operations::Simod => *cell = osa_distance(cell, comparand).to_string(),
             Operations::Eudex => {
+                // safety: we set EUDEX_COMPARAND_HASH in validate_operations()
                 let eudex_comparand_hash = EUDEX_COMPARAND_HASH.get().unwrap();
                 let cell_hash = Hash::new(cell);
                 *cell = format!("{}", (cell_hash - *eudex_comparand_hash).similar());
             },
             Operations::Sentiment => {
+                // safety: we set SENTIMENT_ANALYZER in validate_operations()
                 let sentiment_analyzer = SENTIMENT_ANALYZER
                     .get_or_init(vader_sentiment::SentimentIntensityAnalyzer::new);
                 let sentiment_scores = sentiment_analyzer.polarity_scores(cell);
@@ -1228,6 +1170,7 @@ fn apply_operations(
             Operations::Whatlang => {
                 let lang_info = detect(cell);
                 if let Some(lang_info) = lang_info {
+                    // safety: we set WHATLANG_CONFIDENCE_THRESHOLD in validate_operations()
                     let whatlang_confidence_threshold =
                         *WHATLANG_CONFIDENCE_THRESHOLD.get().unwrap();
                     let lang_confidence = lang_info.confidence();
