@@ -56,7 +56,7 @@ Excel options:
                                Negative indices start from the end (-1 = last sheet). 
                                If the sheet cannot be found, qsv will read the first sheet.
                                [default: 0]
-    --metadata <c|j|J|s>       Outputs workbook metadata in CSV or JSON format:
+    --metadata <c|s|j|J|S>     Outputs workbook metadata in CSV or JSON format:
                                  index, sheet_name, headers, type, visible, num_columns, num_rows,
                                  safe_headers, safe_headers_count, unsafe_headers, unsafe_headers_count
                                  and duplicate_headers_count.
@@ -69,16 +69,29 @@ Excel options:
                                duplicate_headers_count is a count of duplicate header names.
 
                                In CSV(c) mode, the output is in CSV format.
-                               In short CSV(s) mode, the output is in CSV format with only the
+                               In short(s) CSV mode, the output is in CSV format with only the
                                index, sheet_name, type and visible fields.
                                
                                In JSON(j) mode, the output is minified JSON.
                                In Pretty JSON(J) mode, the output is pretty-printed JSON.
-                               For both JSON modes, the filename, the full file path, the workbook format
+                               In Short(S) JSON mode, the output is minified JSON with only the
+                                 index, sheet_name, type and visible fields.
+                               For all JSON modes, the filename, the full file path, the workbook format
                                and the number of sheets are also included.
+                               If metadata retrieval performance is a concern, use the short modes
+                               as they return instantaneously as they don't need to process the sheet data.
                                
                                All other Excel options are ignored.
                                [default: none]
+    --error-format <format>    The format to use when formatting error cells.
+                               There are 3 formats:
+                                 - "code": return the error code.
+                                    (#DIV/0!; #N/A; #NAME?; #NULL!; #NUM!; #REF!; #VALUE!; #DATA!)
+                                 - "formula": return the formula, prefixed with '#'.
+                                    (e.g. #=A1/B1 where B1 is 0; #=100/0)
+                                 - "both": return both error code and the formula.
+                                    (e.g. #DIV/0!: =A1/B1)
+                               [default: code]
     --flexible                 Continue even if the number of columns is different from row to row.
     --trim                     Trim all fields so that leading & trailing whitespaces are removed.
                                Also removes embedded linebreaks.
@@ -106,7 +119,7 @@ use std::{cmp, fmt::Write, path::PathBuf};
 use calamine::{open_workbook, Data, Error, Range, Reader, SheetType, Sheets};
 use indicatif::HumanCount;
 use log::info;
-use rayon::prelude::*;
+use rayon::prelude::{ParallelIterator, ParallelSlice};
 use serde::{Deserialize, Serialize};
 
 use crate::{
@@ -119,6 +132,7 @@ struct Args {
     arg_input:           String,
     flag_sheet:          String,
     flag_metadata:       String,
+    flag_error_format:   String,
     flag_flexible:       bool,
     flag_trim:           bool,
     flag_output:         Option<String>,
@@ -136,9 +150,17 @@ enum MetadataMode {
     ShortCsv,
     Json,
     PrettyJSON,
+    ShortJSON,
     None,
 }
 
+#[derive(PartialEq)]
+enum ErrorFormat {
+    Code,
+    Formula,
+    Both,
+}
+
 #[derive(Serialize, Deserialize)]
 struct SheetMetadata {
     index:                   usize,
@@ -155,6 +177,14 @@ struct SheetMetadata {
     duplicate_headers_count: usize,
 }
 
+#[derive(Serialize, Deserialize)]
+struct ShortSheetMetadata {
+    index:   usize,
+    name:    String,
+    typ:     String,
+    visible: String,
+}
+
 impl From<calamine::Error> for CliError {
     fn from(e: calamine::Error) -> Self {
         CliError::Other(format!("{e}"))
@@ -170,6 +200,15 @@ struct MetadataStruct {
     sheet:              Vec<SheetMetadata>,
 }
 
+#[derive(Serialize, Deserialize)]
+struct ShortMetadataStruct {
+    filename:           String,
+    canonical_filename: String,
+    format:             String,
+    num_sheets:         usize,
+    sheet:              Vec<ShortSheetMetadata>,
+}
+
 struct RequestedRange {
     // matches args for https://docs.rs/calamine/latest/calamine/struct.Range.html#method.rows
     start: (u32, u32), // upper left, 0 based, row, column
@@ -273,9 +312,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let metadata_mode = match first_letter {
         'n' | 'N' => MetadataMode::None,
         'c' | 'C' => MetadataMode::Csv,
-        's' | 'S' => MetadataMode::ShortCsv,
+        's' => MetadataMode::ShortCsv,
         'j' => MetadataMode::Json,
         'J' => MetadataMode::PrettyJSON,
+        'S' => MetadataMode::ShortJSON,
         _ => {
             return fail_incorrectusage_clierror!("Invalid metadata mode: {}", args.flag_metadata);
         },
@@ -284,8 +324,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // check if we're exporting workbook metadata only
     if metadata_mode != MetadataMode::None {
         let mut excelmetadata_struct = MetadataStruct {
-            filename,
-            canonical_filename,
+            filename: filename.clone(),
+            canonical_filename: canonical_filename.clone(),
             format: if ods_flag {
                 "ODS".to_string()
             } else {
@@ -298,7 +338,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let sheet_vec = sheet_names;
 
         for (i, sheet_name) in sheet_vec.iter().enumerate() {
-            let range = if metadata_mode == MetadataMode::ShortCsv {
+            let range = if metadata_mode == MetadataMode::ShortCsv
+                || metadata_mode == MetadataMode::ShortJSON
+            {
                 Range::empty()
             } else if let Some(result) = workbook.worksheet_range_at(i) {
                 match result {
@@ -317,7 +359,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             };
 
             let (header_vec, num_columns, num_rows, safenames_vec, unsafeheaders_vec, dupe_count) =
-                if metadata_mode == MetadataMode::ShortCsv || range.is_empty() {
+                if range.is_empty() {
                     (vec![], 0_usize, 0_usize, vec![], vec![], 0_usize)
                 } else {
                     let (num_rows, num_columns) = range.get_size();
@@ -454,6 +496,28 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 };
                 println!("{json_result}");
             },
+            MetadataMode::ShortJSON => {
+                let mut short_excelmetadata_struct = ShortMetadataStruct {
+                    filename,
+                    canonical_filename,
+                    format,
+                    num_sheets,
+                    sheet: vec![],
+                };
+                for sheetmetadata in excelmetadata_struct.sheet {
+                    let short_sheetmetadata = ShortSheetMetadata {
+                        index:   sheetmetadata.index,
+                        name:    sheetmetadata.name,
+                        typ:     sheetmetadata.typ,
+                        visible: sheetmetadata.visible,
+                    };
+                    short_excelmetadata_struct.sheet.push(short_sheetmetadata);
+                }
+                let Ok(json_result) = serde_json::to_string(&short_excelmetadata_struct) else {
+                    return fail!("Cannot create short JSON");
+                };
+                println!("{json_result}");
+            },
             MetadataMode::None => {},
         }
         info!(r#"exported metadata for "{path}" workbook sheets: {sheet_vec:?}"#);
@@ -551,12 +615,31 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let (row_count, col_count) = range.get_size();
 
+    let error_format = match args.flag_error_format.to_lowercase().as_str() {
+        "formula" => ErrorFormat::Formula,
+        "both" => ErrorFormat::Both,
+        _ => ErrorFormat::Code,
+    };
+
     if row_count == 0 {
+        if !requested_range.is_empty() {
+            return fail_clierror!("\"{requested_range}\" range in sheet \"{sheet}\" is empty.");
+        }
         return fail_clierror!("\"{sheet}\" sheet is empty.");
     }
     // there are rows to export
     let mut rows_iter = range.rows();
 
+    let range_start = range.start().unwrap_or((0, 0));
+
+    // get the sheet formulas only if we need them
+    // as this is an expensive operation
+    let sheet_formulas = if error_format == ErrorFormat::Code {
+        Range::empty()
+    } else {
+        workbook.worksheet_formula(&sheet)?
+    };
+
     // amortize allocations
     let mut record = csv::StringRecord::with_capacity(500, col_count);
     let mut trimmed_record = csv::StringRecord::with_capacity(500, col_count);
@@ -600,9 +683,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     wtr.write_record(&record)?;
 
     let no_date_format: bool;
-    // TODO: Clippy lint is broken, remove allow once fixed.
-    // https://github.com/rust-lang/rust-clippy/issues/12580
-    #[allow(clippy::manual_unwrap_or_default)]
     let date_format = if let Some(df) = args.flag_date_format {
         no_date_format = false;
         df
@@ -611,11 +691,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         String::new()
     };
 
-    let mut rows = Vec::with_capacity(row_count);
+    let mut rows: Vec<(u32, &[Data])> = Vec::with_capacity(row_count);
 
+    // we add 1 as we already processed the header row
+    let mut row_idx = range_start.0 + 1;
     // queue rest of the rows for processing as data rows
     for row in rows_iter {
-        rows.push(row);
+        rows.push((row_idx, row));
+        row_idx += 1;
     }
 
     // set RAYON_NUM_THREADS
@@ -638,8 +721,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let mut formatted_date = String::new();
 
             let mut processed_chunk: Vec<csv::StringRecord> = Vec::with_capacity(chunk_size);
+            let mut col_idx = 0_u32;
 
-            for row in chunk {
+            let formula_get_value_error = "cannot get formula".to_string();
+            let mut cell_value: &String;
+
+            for (row_idx, row) in chunk {
                 for cell in *row {
                     match *cell {
                         Data::Empty => record.push_field(""),
@@ -707,13 +794,28 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
                             record.push_field(&work_date);
                         },
-                        Data::Error(ref e) => record.push_field(&format!("{e}")),
                         Data::Bool(ref b) => {
                             record.push_field(if *b { "true" } else { "false" });
                         },
                         Data::DateTimeIso(ref dt) => record.push_field(dt),
                         Data::DurationIso(ref d) => record.push_field(d),
+                        Data::Error(ref e) => {
+                            match error_format {
+                                ErrorFormat::Code => record.push_field(&format!("{e}")),
+                                ErrorFormat::Formula | ErrorFormat::Both => {
+                                    cell_value = sheet_formulas
+                                        .get_value((*row_idx, col_idx))
+                                        .unwrap_or(&formula_get_value_error);
+                                    if error_format == ErrorFormat::Formula {
+                                        record.push_field(&format!("#={cell_value}"));
+                                    } else {
+                                        record.push_field(&format!("{e}: ={cell_value}"));
+                                    }
+                                },
+                            };
+                        },
                     };
+                    col_idx += 1;
                 }
 
                 if trim {
