@@ -53,7 +53,7 @@ Common options:
                            Must be a single character. (default: ,)
 "#;
 
-use std::{collections::hash_map::Entry, fmt, fs, io, str};
+use std::{collections::hash_map::Entry, fs, io, str};
 
 use ahash::AHashMap;
 use byteorder::{BigEndian, WriteBytesExt};
@@ -243,6 +243,10 @@ impl<R: io::Read + io::Seek> ValueIndex<R> {
     }
 }
 
+#[cfg(debug_assertions)]
+use std::fmt;
+
+#[cfg(debug_assertions)]
 impl<R> fmt::Debug for ValueIndex<R> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         // Sort the values by order of first appearance.
