@@ -140,7 +140,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             state.write_headers()?;
             state.inner_join()
         },
-        _ => fail!("Please pick exactly one join operation."),
+        _ => fail_incorrectusage_clierror!("Please pick exactly one join operation."),
     }
 }
 
@@ -190,7 +190,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
                 }
             }
         }
-        Ok(())
+        Ok(self.wtr.flush()?)
     }
 
     fn outer_join(mut self, right: bool) -> CliResult<()> {
@@ -223,7 +223,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
                 self.wtr.write_record(row.iter().chain(&pad2))?;
             }
         }
-        Ok(())
+        Ok(self.wtr.flush()?)
     }
 
     fn left_join(mut self, anti: bool) -> CliResult<()> {
@@ -240,7 +240,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
                 self.wtr.write_record(&row)?;
             }
         }
-        Ok(())
+        Ok(self.wtr.flush()?)
     }
 
     fn full_outer_join(mut self) -> CliResult<()> {
@@ -276,7 +276,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
                 self.wtr.write_record(pad1.iter().chain(&scratch))?;
             }
         }
-        Ok(())
+        Ok(self.wtr.flush()?)
     }
 
     fn cross_join(mut self) -> CliResult<()> {
@@ -296,7 +296,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
                 self.wtr.write_record(row1.iter().chain(&row2))?;
             }
         }
-        Ok(())
+        Ok(self.wtr.flush()?)
     }
 
     fn get_padding(&mut self) -> CliResult<(csv::ByteRecord, csv::ByteRecord)> {
@@ -350,7 +350,7 @@ impl Args {
         let select1 = rconf1.selection(headers1)?;
         let select2 = rconf2.selection(headers2)?;
         if select1.len() != select2.len() {
-            return fail_clierror!(
+            return fail_incorrectusage_clierror!(
                 "Column selections must have the same number of columns, but found column \
                  selections with {} and {} columns.",
                 select1.len(),
