@@ -66,20 +66,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let headers = rdr.byte_headers()?;
 
     if args.arg_headers.to_lowercase() == "_all_generic" {
-        let mut generic_headers = String::new();
-        for (i, _) in headers.iter().enumerate() {
-            generic_headers.push_str(&format!("_col_{},", i + 1));
-        }
-        // remove the trailing comma
-        generic_headers.pop();
-        args.arg_headers = generic_headers;
+        args.arg_headers = rename_headers_all_generic(headers.len());
     }
 
     let mut new_rdr = csv::Reader::from_reader(args.arg_headers.as_bytes());
     let new_headers = new_rdr.byte_headers()?;
 
     if headers.len() != new_headers.len() {
-        return fail_clierror!(
+        return fail_incorrectusage_clierror!(
             "The length of the CSV headers ({}) is different from the provided one ({}).",
             headers.len(),
             new_headers.len()
@@ -92,6 +86,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     while rdr.read_byte_record(&mut record)? {
         wtr.write_record(&record)?;
     }
-    wtr.flush()?;
-    Ok(())
+    Ok(wtr.flush()?)
+}
+
+pub fn rename_headers_all_generic(num_of_cols: usize) -> String {
+    let mut generic_headers = String::new();
+    for i in 1..=num_of_cols {
+        generic_headers.push_str(&format!("_col_{i},"));
+    }
+    // remove the trailing comma
+    generic_headers.pop();
+    generic_headers
 }
