@@ -46,44 +46,39 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers);
 
-    // quadruple the buffer sizes as this is IO intensive
-    // this is our brute force way of speeding up the process
-    // as we can't really parallelize reversing a file
-    let mut rdr = rconfig
-        .clone()
-        .read_buffer(rconfig.read_buffer * 4)
-        .reader()?;
-    let mut wtr = Config::new(&args.flag_output)
-        .write_buffer(rconfig.write_buffer * 4)
-        .writer()?;
+    let mut rdr = rconfig.reader()?;
+    let mut wtr = Config::new(&args.flag_output).writer()?;
 
-    let Some(mut idx_file) = rconfig.indexed()? else {
-        // we don't have an index, we need to read the entire file into memory
-        // we're loading the entire file into memory, we need to check avail mem
-        if let Some(ref path) = rconfig.path {
-            util::mem_file_check(path, false, args.flag_memcheck)?;
-        }
+    #[allow(clippy::single_match_else)]
+    match rconfig.indexed()? {
+        Some(mut idx_file) => {
+            // we have an index, no need to check avail mem,
+            // we're reading the file in reverse streaming
+            rconfig.write_headers(&mut rdr, &mut wtr)?;
+            let mut record = csv::ByteRecord::new();
+            let mut pos = idx_file.count().saturating_sub(1);
+            idx_file.seek(pos)?;
+            while idx_file.read_byte_record(&mut record)? {
+                wtr.write_byte_record(&record)?;
+                pos -= 1;
+                idx_file.seek(pos)?; // seek to next pos
+            }
+        },
+        None => {
+            // we don't have an index, we need to read the entire file into memory
+            // we're loading the entire file into memory, we need to check avail mem
+            if let Some(ref path) = rconfig.path {
+                util::mem_file_check(path, false, args.flag_memcheck)?;
+            }
 
-        let mut all = rdr.byte_records().collect::<Result<Vec<_>, _>>()?;
-        all.reverse();
+            let mut all = rdr.byte_records().collect::<Result<Vec<_>, _>>()?;
+            all.reverse();
 
-        rconfig.write_headers(&mut rdr, &mut wtr)?;
-        for r in all {
-            wtr.write_byte_record(&r)?;
-        }
-        return Ok(wtr.flush()?);
-    };
-
-    // we have an index, no need to check avail mem,
-    // we're reading the file in reverse streaming
-    rconfig.write_headers(&mut rdr, &mut wtr)?;
-    let mut record = csv::ByteRecord::new();
-    let mut pos = idx_file.count().saturating_sub(1);
-    idx_file.seek(pos)?;
-    while idx_file.read_byte_record(&mut record)? {
-        wtr.write_byte_record(&record)?;
-        pos -= 1;
-        idx_file.seek(pos)?; // seek to next pos
+            rconfig.write_headers(&mut rdr, &mut wtr)?;
+            for r in all {
+                wtr.write_byte_record(&r)?;
+            }
+        },
     }
 
     Ok(wtr.flush()?)
