@@ -2,11 +2,12 @@ static USAGE: &str = r#"
 Run blazing-fast Polars SQL queries against several CSVs - replete with joins, aggregations,
 grouping, table functions, sorting, and more - working on larger than memory CSV files.
 
-Polars SQL is a SQL dialect, converting SQL queries to fast Polars LazyFrame expressions.
+Polars SQL is a SQL dialect, converting SQL queries to fast Polars LazyFrame expressions
+(see https://docs.pola.rs/user-guide/sql/intro/).
 
 For a list of SQL functions and keywords supported by Polars SQL, see
-https://github.com/pola-rs/polars/blob/rs-0.38.3/crates/polars-sql/src/functions.rs
-https://github.com/pola-rs/polars/blob/rs-0.38.3/crates/polars-sql/src/keywords.rs and
+https://github.com/pola-rs/polars/blob/rs-0.39.2/crates/polars-sql/src/functions.rs
+https://github.com/pola-rs/polars/blob/rs-0.39.2/crates/polars-sql/src/keywords.rs and
 https://github.com/pola-rs/polars/issues/7227
 
 Returns the shape of the query result (number of rows, number of columns) to stderr.
@@ -47,6 +48,11 @@ Example queries:
   # When running several queries, each query needs to be separated by a semicolon,
   # the last query will be returned as the result.
   # Typically, earlier queries are used to create tables that can be used in later queries.
+  # Note that scripts support single-line comments starting with '--' so feel free to
+  # add comments to your script.
+  # In long, complex scripts that produce multiple temporary tables, note that you can use
+  # `truncate table <table_name>;` to free up memory used by temporary tables. Otherwise,
+  # the memory used by the temporary tables won't be freed until the script finishes.
   # See test_sqlp/sqlp_boston311_sql_script() for an example.
    qsv sqlp data.csv data2.csv data3.csv data4.csv script.sql --format json --output data.json
 
@@ -122,6 +128,9 @@ sqlp arguments:
                            decompressed automatically.
                            Column headers are required. Use 'qsv rename _all_generic --no-headers'
                            to add generic column names (_col_N) to a CSV with no headers.
+                           If you are using Polars SQL's table functions like read_csv() & read_parquet()
+                           to read input files directly, you can use the special value 'SKIP_INPUT'
+                           to skip input preprocessing.
 
     sql                    The SQL query/ies to run. Each input file will be available as a table
                            named after the file name (without the extension), or as "_t_N"
@@ -129,6 +138,7 @@ sqlp arguments:
                            If the query ends with ".sql", it will be read as a SQL script file,
                            with each SQL query separated by a semicolon. It will execute the queries
                            in order, and the result of the LAST query will be returned as the result.
+                           SQL scripts support single-line comments starting with '--'.
 
 sqlp options:
     --format <arg>            The output format to use. Valid values are:
@@ -138,7 +148,7 @@ sqlp options:
                                 parquet  Apache Parquet
                                 arrow    Apache Arrow IPC
                                 avro     Apache Avro
-                              (default: csv)
+                              [default: csv]
 
                               POLARS CSV INPUT PARSING OPTIONS:
     --try-parsedates          Automatically try to parse dates/datetimes and time.
@@ -148,7 +158,7 @@ sqlp options:
                               the CSV parsing of dates.
     --infer-len <arg>         The number of rows to scan when inferring the schema of the CSV.
                               Set to 0 to do a full table scan (warning: very slow).
-                              (default: 250)
+                              [default: 1000]
     --low-memory              Use low memory mode when parsing CSVs. This will use less memory
                               but will be slower. It will also process LazyFrames in streaming mode.
                               Only use this when you get out of memory errors.
@@ -169,7 +179,7 @@ sqlp options:
     --rnull-values <arg>      The comma-delimited list of case-sensitive strings to consider as
                               null values when READING CSV files (e.g. NULL, NONE, <empty string>).
                               Use "<empty string>" to consider an empty string a null value.
-                              (default: <empty string>)
+                              [default: <empty string>]
 
                               CSV OUTPUT FORMAT ONLY:
     --datetime-format <fmt>   The datetime format to use writing datetimes.
@@ -178,16 +188,15 @@ sqlp options:
     --date-format <fmt>       The date format to use writing dates.
     --time-format <fmt>       The time format to use writing times.
     --float-precision <arg>   The number of digits of precision to use when writing floats.
-                              (default: 6)
     --wnull-value <arg>       The string to use when WRITING null values.
-                              (default: <empty string>)
+                              [default: <empty string>]
 
                               ARROW/AVRO/PARQUET OUTPUT FORMATS ONLY:
     --compression <arg>       The compression codec to use when writing arrow or parquet files.
                                 For Arrow, valid values are: zstd, lz4, uncompressed
                                 For Avro, valid values are: deflate, snappy, uncompressed (default)
                                 For Parquet, valid values are: zstd, lz4raw, gzip, snappy, uncompressed
-                              (default: zstd)
+                              [default: zstd]
 
                               PARQUET OUTPUT FORMAT ONLY:
     --compress-level <arg>    The compression level to use when using zstd or gzip compression.
@@ -203,7 +212,7 @@ Common options:
     -h, --help             Display this message
     -o, --output <file>    Write output to <file> instead of stdout.
     -d, --delimiter <arg>  The field delimiter for reading and writing CSV data.
-                           Must be a single character. (default: ,)
+                           Must be a single character. [default: ,]
     -Q, --quiet            Do not return result shape to stderr.
 "#;
 
@@ -248,7 +257,7 @@ struct Args {
     arg_sql:                    String,
     flag_format:                String,
     flag_try_parsedates:        bool,
-    flag_infer_len:             Option<usize>,
+    flag_infer_len:             usize,
     flag_low_memory:            bool,
     flag_no_optimizations:      bool,
     flag_ignore_errors:         bool,
@@ -491,7 +500,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut args: Args = util::get_args(USAGE, argv)?;
 
     let tmpdir = tempfile::tempdir()?;
-    args.arg_input = process_input(args.arg_input, &tmpdir, "")?;
+
+    let mut skip_input = false;
+    args.arg_input = if args.arg_input == [PathBuf::from_str("SKIP_INPUT").unwrap()] {
+        skip_input = true;
+        Vec::new()
+    } else {
+        process_input(args.arg_input, &tmpdir, "")?
+    };
 
     let rnull_values = if args.flag_rnull_values == "<empty string>" {
         vec![String::new()]
@@ -559,12 +575,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     if debuglog_flag {
         log::debug!("Optimization state: {optimization_state:?}");
         log::debug!(
-            "Delimiter: {delim} Infer_schema_len: {infer_len:?} try_parse_dates: {parse_dates} \
-             ignore_errors: {ignore_errors}, low_memory: {low_memory}",
+            "Delimiter: {delim} Infer_schema_len: {infer_len} try_parse_dates: {parse_dates} \
+             ignore_errors: {ignore_errors}, low_memory: {low_memory}, float_precision: \
+             {float_precision:?}, skip_input: {skip_input}",
             infer_len = args.flag_infer_len,
             parse_dates = args.flag_try_parsedates,
             ignore_errors = args.flag_ignore_errors,
-            low_memory = args.flag_low_memory
+            low_memory = args.flag_low_memory,
+            float_precision = args.flag_float_precision,
         );
     }
 
@@ -581,12 +599,20 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     // if there is only one input file and its a CSV and no CSV parsing options are used,
     // we can use the fast path to read the CSV directly into a LazyFrame without having to
     // parse and register it as a table in the SQL context using Polars SQL's read_csv function
-    if args.arg_input.len() == 1
+    // The user can also skip all these heuristics and force the fast path by using the special
+    // value '<SKIP_INPUT>' as the input file.
+    if skip_input {
+        // we don't need to do anything here, as we are skipping input
+        if debuglog_flag {
+            // Using the slow path to read and parse the CSV/s into tables in the SQL context.
+            log::debug!("Skipping input processing...");
+        }
+    } else if args.arg_input.len() == 1
         && !is_sql_script
         && delim == b','
         && !args.flag_no_optimizations
         && !args.flag_try_parsedates
-        && args.flag_infer_len != Some(250)
+        && args.flag_infer_len != 1000
         && !args.flag_low_memory
         && !args.flag_truncate_ragged_lines
         && !args.flag_ignore_errors
@@ -652,7 +678,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 .with_comment_prefix(comment_char.as_deref())
                 .with_null_values(Some(NullValues::AllColumns(rnull_values.clone())))
                 .with_separator(tsvtab_delim(table, delim))
-                .with_infer_schema_length(args.flag_infer_len)
+                .with_infer_schema_length(Some(args.flag_infer_len))
                 .with_try_parse_dates(args.flag_try_parsedates)
                 .with_ignore_errors(args.flag_ignore_errors)
                 .truncate_ragged_lines(args.flag_truncate_ragged_lines)
@@ -662,7 +688,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         }
     }
 
-    if debuglog_flag {
+    if debuglog_flag && !skip_input {
         let tables_in_context = ctx.get_tables();
         log::debug!("Table(s) registered in SQL Context: {tables_in_context:?}");
     }
@@ -672,6 +698,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let mut file = File::open(&args.arg_sql)?;
         let mut sql_script = String::new();
         file.read_to_string(&mut sql_script)?;
+
+        // remove comments from the SQL script
+        // we only support single-line comments in SQL scripts
+        // i.e. comments that start with "--" and end at the end of the line
+        // so the regex is performant and simple
+        let comment_regex = Regex::new(r"^--.*$")?;
+        let sql_script = comment_regex.replace_all(&sql_script, "");
         sql_script
             .split(';')
             .map(std::string::ToString::to_string)
