@@ -69,8 +69,8 @@ Usage:
 Validate arguments:
     <input>                    Input CSV file to validate. If not provided, will read from stdin.
     <json-schema>              JSON Schema file to validate against. If not provided, `validate`
-                               will run in RFC 4180 validation mode.
-                               The file can be a local file or a URL.
+                               will run in RFC 4180 validation mode. The file can be a local file
+                               or a URL (http and https schemes supported).
 
 Validate options:
     --trim                     Trim leading and trailing whitespace from fields before validating.
@@ -80,13 +80,18 @@ Validate options:
     --json                     When validating without a schema, return the RFC 4180 check
                                as a JSON file instead of a message.
     --pretty-json              Same as --json, but pretty printed.
+    --valid-output <file>      Change validation mode behavior so if ALL rows are valid, to pass it to
+                               output, return exit code 1, and set stderr to the number of valid rows.
+                               Setting this will override the default behavior of creating
+                               a valid file only when there are invalid records.
+                               To send valid records to stdout, use `-` as the filename.
     -j, --jobs <arg>           The number of jobs to run in parallel.
                                When not set, the number of jobs is set to the
                                number of CPUs detected.
     -b, --batch <size>         The number of rows per batch to load into memory,
                                before running in parallel.
                                [default: 50000]
-    --timeout <seconds>        Timeout for downloading json-schemas.
+    --timeout <seconds>        Timeout for downloading json-schemas on URLs.
                                [default: 30]
 
 Common options:
@@ -135,26 +140,27 @@ use crate::{
 // to save on repeated init/allocs
 static NULL_TYPE: OnceLock<Value> = OnceLock::new();
 
-static TIMEOUT_SECS: AtomicU16 = AtomicU16::new(15);
+static TIMEOUT_SECS: AtomicU16 = AtomicU16::new(30);
 
 #[derive(Deserialize)]
 #[allow(dead_code)]
 struct Args {
-    flag_trim:        bool,
-    flag_fail_fast:   bool,
-    flag_valid:       Option<String>,
-    flag_invalid:     Option<String>,
-    flag_json:        bool,
-    flag_pretty_json: bool,
-    flag_jobs:        Option<usize>,
-    flag_batch:       u32,
-    flag_no_headers:  bool,
-    flag_delimiter:   Option<Delimiter>,
-    flag_progressbar: bool,
-    flag_quiet:       bool,
-    arg_input:        Option<String>,
-    arg_json_schema:  Option<String>,
-    flag_timeout:     u16,
+    flag_trim:         bool,
+    flag_fail_fast:    bool,
+    flag_valid:        Option<String>,
+    flag_invalid:      Option<String>,
+    flag_json:         bool,
+    flag_pretty_json:  bool,
+    flag_valid_output: Option<String>,
+    flag_jobs:         Option<usize>,
+    flag_batch:        u32,
+    flag_no_headers:   bool,
+    flag_delimiter:    Option<Delimiter>,
+    flag_progressbar:  bool,
+    flag_quiet:        bool,
+    arg_input:         Option<String>,
+    arg_json_schema:   Option<String>,
+    flag_timeout:      u16,
 }
 
 enum JSONtypes {
@@ -364,7 +370,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 break 'rfc4180_check;
             }
             record_idx += 1;
-        }
+        } // end rfc4180_check loop
 
         // if we're here, we know the CSV is valid
         let msg = if flag_json {
@@ -546,11 +552,34 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         util::finish_progress(&progress);
     }
 
-    // only write out invalid/valid/errors output files if there are actually invalid records.
-    // if 100% invalid, valid file is not needed, but this is rare so OK with creating empty file.
-    if invalid_count > 0 {
-        let msg = "Writing invalid/valid/error files...";
-        woutinfo!("{msg}");
+    if invalid_count == 0 {
+        // no invalid records found
+        // see if we need to pass all valid records to output
+        if let Some(valid_output) = args.flag_valid_output {
+            // pass all valid records to output and return exit code 1
+            let valid_path = if valid_output == "-" {
+                // write to stdout
+                None
+            } else {
+                Some(valid_output)
+            };
+
+            let mut valid_wtr = Config::new(&valid_path).writer()?;
+            valid_wtr.write_byte_record(&headers)?;
+
+            let mut rdr = rconfig.reader()?;
+            let mut record = csv::ByteRecord::new();
+            while rdr.read_byte_record(&mut record)? {
+                valid_wtr.write_byte_record(&record)?;
+            }
+            valid_wtr.flush()?;
+            // return 1 as an exitcode and the number of valid rows to stderr
+            return fail_clierror!("{row_number}");
+        }
+    } else {
+        // there are invalid records. write out invalid/valid/errors output files.
+        // if 100% invalid, valid file isn't needed, but this is rare so OK creating empty file.
+        woutinfo!("Writing invalid/valid/error files...");
 
         let input_path = args
             .arg_input
@@ -667,6 +696,7 @@ fn write_error_report(input_path: &str, validation_error_messages: Vec<String>)
 }
 
 /// if given record is valid, return None, otherwise, error file entry string
+#[inline]
 fn do_json_validation(
     header_types: &[(String, JSONtypes)],
     header_len: usize,
@@ -708,46 +738,54 @@ fn to_json_instance(
 ) -> CliResult<Value> {
     let mut json_object_map: Map<String, Value> = Map::with_capacity(header_len);
 
-    for ((key, json_type), value) in header_types.iter().zip(record.iter()) {
+    let mut key_string: String;
+    let mut value_string: String;
+    for ((key_iter, json_type), value) in header_types.iter().zip(record.iter()) {
+        key_string = key_iter.to_owned();
+
         if value.is_empty() {
-            json_object_map.insert(key.to_string(), Value::Null);
+            json_object_map.insert(key_string, Value::Null);
             continue;
         }
 
-        let Ok(value_str) = from_utf8(value) else {
-            let s = String::from_utf8_lossy(value);
-            return fail_encoding_clierror!("CSV value is not valid UTF-8: {s}");
+        let value_str = match from_utf8(value) {
+            Ok(v) => v,
+            Err(e) => {
+                let s = String::from_utf8_lossy(value);
+                return fail_encoding_clierror!("CSV value \"{s}\" is not valid UTF-8: {e}");
+            },
         };
 
         match *json_type {
             JSONtypes::String => {
-                json_object_map.insert(key.clone(), Value::String(value_str.to_owned()));
+                value_string = value_str.to_owned();
+                json_object_map.insert(key_string, Value::String(value_string));
             },
             JSONtypes::Number => {
                 if let Ok(float) = value_str.parse::<f64>() {
                     json_object_map
-                        .insert(key.clone(), Value::Number(Number::from_f64(float).unwrap()));
+                        .insert(key_string, Value::Number(Number::from_f64(float).unwrap()));
                 } else {
                     return fail_clierror!(
-                        "Can't cast into Number. key: {key}, value: {value_str}"
+                        "Can't cast into Number. key: {key_string}, value: {value_str}"
                     );
                 }
             },
             JSONtypes::Integer => {
                 if let Ok(int) = atoi_simd::parse::<i64>(value_str.as_bytes()) {
-                    json_object_map.insert(key.clone(), Value::Number(Number::from(int)));
+                    json_object_map.insert(key_string, Value::Number(Number::from(int)));
                 } else {
                     return fail_clierror!(
-                        "Can't cast into Integer. key: {key}, value: {value_str}"
+                        "Can't cast into Integer. key: {key_string}, value: {value_str}"
                     );
                 }
             },
             JSONtypes::Boolean => {
                 if let Ok(boolean) = value_str.parse::<bool>() {
-                    json_object_map.insert(key.clone(), Value::Bool(boolean));
+                    json_object_map.insert(key_string, Value::Bool(boolean));
                 } else {
                     return fail_clierror!(
-                        "Can't cast into Boolean. key: {key}, value: {value_str}"
+                        "Can't cast into Boolean. key: {key_string}, value: {value_str}"
                     );
                 }
             },
