@@ -1,12 +1,12 @@
 use std::{
     cmp::Ordering,
-    collections::HashSet,
     fmt,
     iter::{self, repeat},
     ops, slice,
     str::FromStr,
 };
 
+use ahash::AHashSet;
 use regex::bytes::Regex;
 use serde::de::{Deserialize, Deserializer, Error};
 
@@ -52,7 +52,7 @@ impl SelectColumns {
             map.extend(idxs?);
         }
         if self.invert {
-            let set: HashSet<_> = map.into_iter().collect();
+            let set: AHashSet<_> = map.into_iter().collect();
             let mut map = vec![];
             for i in 0..first_record.len() {
                 if !set.contains(&i) {
@@ -63,6 +63,11 @@ impl SelectColumns {
         }
         Ok(Selection(map))
     }
+
+    // commented out because it's unused
+    // pub fn is_empty(&self) -> bool {
+    //     self.selectors.is_empty()
+    // }
 }
 
 impl fmt::Debug for SelectColumns {
