@@ -416,6 +416,7 @@ macro_rules! update_cache_info {
                 if size > 0 {
                     let hits = cache.cache_hits().unwrap_or_default();
                     let misses = cache.cache_misses().unwrap_or(1);
+                    #[allow(clippy::cast_precision_loss)]
                     let hit_ratio = (hits as f64 / (hits + misses) as f64) * 100.0;
                     let capacity = cache.cache_capacity();
                     $progress.set_message(format!(
@@ -433,6 +434,7 @@ macro_rules! update_cache_info {
     ($progress:expr, $cache_hits:expr, $num_rows:expr) => {
         use indicatif::HumanCount;
 
+        #[allow(clippy::cast_precision_loss)]
         let hit_ratio = ($cache_hits as f64 / $num_rows as f64) * 100.0;
         $progress.set_message(format!(
             " of {} records. Cache hit ratio: {hit_ratio:.2}%",
@@ -637,7 +639,7 @@ pub fn range(start: Idx, end: Idx, len: Idx, index: Idx) -> Result<(usize, usize
         (_, Some(_), Some(_), None) => {
             fail!("--end and --len cannot be used at the same time.")
         },
-        (_, None, None, None) => Ok((start.unwrap_or(0), ::std::usize::MAX)),
+        (_, None, None, None) => Ok((start.unwrap_or(0), usize::MAX)),
         (_, Some(e), None, None) => {
             let s = start.unwrap_or(0);
             if s > e {
@@ -740,7 +742,7 @@ impl<'de> Deserialize<'de> for FilenameTemplate {
     }
 }
 
-pub fn init_logger() -> (String, flexi_logger::LoggerHandle) {
+pub fn init_logger() -> CliResult<(String, flexi_logger::LoggerHandle)> {
     use flexi_logger::{Cleanup, Criterion, FileSpec, Logger, Naming};
 
     let qsv_log_env = env::var("QSV_LOG_LEVEL").unwrap_or_else(|_| "off".to_string());
@@ -751,32 +753,36 @@ pub fn init_logger() -> (String, flexi_logger::LoggerHandle) {
         flexi_logger::WriteMode::BufferAndFlush
     };
 
-    let logger = Logger::try_with_env_or_str(qsv_log_env)
-        .unwrap()
-        .use_utc()
-        .log_to_file(
-            FileSpec::default()
-                .directory(qsv_log_dir)
-                .suppress_timestamp(),
-        )
-        .write_mode(write_mode)
-        .format_for_files(flexi_logger::detailed_format)
-        .o_append(true)
-        .rotate(
-            Criterion::Size(20_000_000), // 20 mb
-            Naming::Numbers,
-            Cleanup::KeepLogAndCompressedFiles(10, 100),
-        )
-        .start()
-        .unwrap();
+    let logger_handle = Logger::try_with_env_or_str(qsv_log_env);
 
-    let qsv_args: String = if log::log_enabled!(log::Level::Info) {
-        env::args().skip(1).collect::<Vec<_>>().join(" ")
-    } else {
-        String::new()
-    };
-    log::info!("START: {qsv_args}");
-    (qsv_args, logger)
+    match logger_handle {
+        Ok(logger) => {
+            let logger = logger
+                .use_utc()
+                .log_to_file(
+                    FileSpec::default()
+                        .directory(qsv_log_dir)
+                        .suppress_timestamp(),
+                )
+                .write_mode(write_mode)
+                .format_for_files(flexi_logger::detailed_format)
+                .o_append(true)
+                .rotate(
+                    Criterion::Size(20_000_000), // 20 mb
+                    Naming::Numbers,
+                    Cleanup::KeepLogAndCompressedFiles(10, 100),
+                )
+                .start()?;
+            let qsv_args: String = if log_enabled!(log::Level::Info) {
+                env::args().skip(1).collect::<Vec<_>>().join(" ")
+            } else {
+                String::new()
+            };
+            log::info!("START: {qsv_args}");
+            Ok((qsv_args, logger))
+        },
+        Err(e) => Err(CliError::Other(format!("Failed to initialize logger: {e}"))),
+    }
 }
 
 #[cfg(feature = "self_update")]
@@ -1367,7 +1373,7 @@ pub fn decompress_snappy_file(
 pub async fn download_file(
     url: &str,
     path: PathBuf,
-    show_progress: bool,
+    #[allow(unused_variables)] show_progress: bool,
     custom_user_agent: Option<String>,
     download_timeout: Option<u16>,
     sample_size: Option<u64>,
@@ -1390,7 +1396,7 @@ pub async fn download_file(
         .use_rustls_tls()
         .http2_adaptive_window(true)
         .connection_verbose(log_enabled!(log::Level::Debug) || log_enabled!(log::Level::Trace))
-        .timeout(download_timeout)
+        .read_timeout(download_timeout)
         .build()
     {
         Ok(c) => c,
