@@ -36,6 +36,80 @@ fn excel_cellerrors() {
         svec!["3", "50", "20"],
         svec!["4", "33.333333333333336", "3"],
         svec!["5", "25", "4"],
+        svec!["#VALUE!", "#VALUE!", "#VALUE!"],
+        svec!["7", "20", "#NAME?"],
+        svec!["8", "Hello", "hello"],
+        svec!["9", "abcd", "wxyz"],
+    ];
+    assert_eq!(got, expected);
+}
+
+// for now, only run this test on macos on aarch64
+// since we have self-hosted runners on macos on aarch64
+// For some reason, The test doesn't work on GitHub's Action Runners
+// but works when run locally on native hardware. Could be an
+// issue with the GitHub's Action Runners running on Azure VMs
+#[cfg(all(target_os = "macos", target_arch = "aarch64"))]
+#[test]
+fn excel_cellerrors_formula() {
+    let wrk = Workdir::new("excel_cellerrors_formula");
+
+    let xls_file = wrk.load_test_file("excel-xlsx.xlsx");
+
+    let mut cmd = wrk.command("excel");
+    cmd.args(["--sheet", "cellerrors"])
+        .args(["--error-format", "formula"])
+        .arg(xls_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["col1", "col 2", "column-3"],
+        svec!["1", "-50", "15"],
+        svec!["2", "#=100/0", "#=4*te"],
+        svec!["3", "50", "20"],
+        svec!["4", "33.333333333333336", "3"],
+        svec!["5", "25", "4"],
+        svec![
+            "#=B7+12",
+            "#=C7+20",
+            "#=_xlfn._xlws.SORT(_xlfn.CHOOSECOLS(A3:B20, 3))"
+        ],
+        svec!["7", "20", "#=SUM(C2:C7)"],
+        svec!["8", "Hello", "hello"],
+        svec!["9", "abcd", "wxyz"],
+    ];
+    assert_eq!(got, expected);
+}
+
+// same as above, only run this test on macos/Apple Silicon
+#[cfg(all(target_os = "macos", target_arch = "aarch64"))]
+#[test]
+fn excel_cellerrors_both() {
+    let wrk = Workdir::new("excel_cellerrors_both");
+
+    let xls_file = wrk.load_test_file("excel-xlsx.xlsx");
+
+    let mut cmd = wrk.command("excel");
+    cmd.args(["--sheet", "cellerrors"])
+        .args(["--error-format", "both"])
+        .arg(xls_file);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["col1", "col 2", "column-3"],
+        svec!["1", "-50", "15"],
+        svec!["2", "#DIV/0!: =100/0", "#NAME?: =4*te"],
+        svec!["3", "50", "20"],
+        svec!["4", "33.333333333333336", "3"],
+        svec!["5", "25", "4"],
+        svec![
+            "#VALUE!: =B7+12",
+            "#VALUE!: =C7+20",
+            "#VALUE!: =_xlfn._xlws.SORT(_xlfn.CHOOSECOLS(A3:B20, 3))"
+        ],
+        svec!["7", "20", "#NAME?: =SUM(C2:C7)"],
+        svec!["8", "Hello", "hello"],
+        svec!["9", "abcd", "wxyz"],
     ];
     assert_eq!(got, expected);
 }
@@ -1089,6 +1163,21 @@ fn excel_metadata_sheet_types_xlsx() {
     wrk.assert_success(&mut cmd);
 }
 
+#[test]
+fn excel_metadata_sheet_types_xlsx_short_json() {
+    let wrk = Workdir::new("excel_metadata_sheet_types_xlsx_short_json");
+
+    let xlsx_file = wrk.load_test_file("any_sheets.xlsx");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg("--metadata").arg("S").arg(xlsx_file);
+
+    let got: String = wrk.stdout(&mut cmd);
+    let expected = r#"any_sheets.xlsx","format":"xlsx","num_sheets":4,"sheet":[{"index":0,"name":"Visible","typ":"WorkSheet","visible":"Visible"},{"index":1,"name":"Hidden","typ":"WorkSheet","visible":"Hidden"},{"index":2,"name":"VeryHidden","typ":"WorkSheet","visible":"VeryHidden"},{"index":3,"name":"Chart","typ":"ChartSheet","visible":"Visible"}]}"#;
+    assert!(got.ends_with(expected));
+    wrk.assert_success(&mut cmd);
+}
+
 #[test]
 fn excel_metadata_sheet_types_xlsb() {
     let wrk = Workdir::new("excel_metadata_sheet_types_xlsb");
